/****************************************************************************
** Meta object code from reading C++ file 'Waveform.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../Waveform.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Waveform.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Waveform_t {
    QByteArrayData data[35];
    char stringdata0[547];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Waveform_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Waveform_t qt_meta_stringdata_Waveform = {
    {
QT_MOC_LITERAL(0, 0, 8), // "Waveform"
QT_MOC_LITERAL(1, 9, 22), // "backgroundColorChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 22), // "foregroundColorChanged"
QT_MOC_LITERAL(4, 56, 16), // "axesColorChanged"
QT_MOC_LITERAL(5, 73, 16), // "textColorChanged"
QT_MOC_LITERAL(6, 90, 22), // "horizontalUnitsChanged"
QT_MOC_LITERAL(7, 113, 20), // "verticalUnitsChanged"
QT_MOC_LITERAL(8, 134, 21), // "selectionColorChanged"
QT_MOC_LITERAL(9, 156, 24), // "subSelectionColorChanged"
QT_MOC_LITERAL(10, 181, 20), // "envelopeColorChanged"
QT_MOC_LITERAL(11, 202, 17), // "sampleRateChanged"
QT_MOC_LITERAL(12, 220, 11), // "axesUpdated"
QT_MOC_LITERAL(13, 232, 16), // "selectionUpdated"
QT_MOC_LITERAL(14, 249, 22), // "horizontalRangeChanged"
QT_MOC_LITERAL(15, 272, 5), // "start"
QT_MOC_LITERAL(16, 278, 3), // "end"
QT_MOC_LITERAL(17, 282, 20), // "verticalRangeChanged"
QT_MOC_LITERAL(18, 303, 3), // "min"
QT_MOC_LITERAL(19, 307, 3), // "max"
QT_MOC_LITERAL(20, 311, 26), // "horizontalSelectionChanged"
QT_MOC_LITERAL(21, 338, 24), // "verticalSelectionChanged"
QT_MOC_LITERAL(22, 363, 9), // "hoverTime"
QT_MOC_LITERAL(23, 373, 15), // "waveViewChanged"
QT_MOC_LITERAL(24, 389, 17), // "onWaveViewChanges"
QT_MOC_LITERAL(25, 407, 15), // "backgroundColor"
QT_MOC_LITERAL(26, 423, 15), // "foregroundColor"
QT_MOC_LITERAL(27, 439, 9), // "axesColor"
QT_MOC_LITERAL(28, 449, 9), // "textColor"
QT_MOC_LITERAL(29, 459, 14), // "selectionColor"
QT_MOC_LITERAL(30, 474, 17), // "subSelectionColor"
QT_MOC_LITERAL(31, 492, 13), // "envelopeColor"
QT_MOC_LITERAL(32, 506, 10), // "sampleRate"
QT_MOC_LITERAL(33, 517, 15), // "horizontalUnits"
QT_MOC_LITERAL(34, 533, 13) // "verticalUnits"

    },
    "Waveform\0backgroundColorChanged\0\0"
    "foregroundColorChanged\0axesColorChanged\0"
    "textColorChanged\0horizontalUnitsChanged\0"
    "verticalUnitsChanged\0selectionColorChanged\0"
    "subSelectionColorChanged\0envelopeColorChanged\0"
    "sampleRateChanged\0axesUpdated\0"
    "selectionUpdated\0horizontalRangeChanged\0"
    "start\0end\0verticalRangeChanged\0min\0"
    "max\0horizontalSelectionChanged\0"
    "verticalSelectionChanged\0hoverTime\0"
    "waveViewChanged\0onWaveViewChanges\0"
    "backgroundColor\0foregroundColor\0"
    "axesColor\0textColor\0selectionColor\0"
    "subSelectionColor\0envelopeColor\0"
    "sampleRate\0horizontalUnits\0verticalUnits"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Waveform[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
      10,  146, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      18,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x06 /* Public */,
       3,    0,  110,    2, 0x06 /* Public */,
       4,    0,  111,    2, 0x06 /* Public */,
       5,    0,  112,    2, 0x06 /* Public */,
       6,    0,  113,    2, 0x06 /* Public */,
       7,    0,  114,    2, 0x06 /* Public */,
       8,    0,  115,    2, 0x06 /* Public */,
       9,    0,  116,    2, 0x06 /* Public */,
      10,    0,  117,    2, 0x06 /* Public */,
      11,    0,  118,    2, 0x06 /* Public */,
      12,    0,  119,    2, 0x06 /* Public */,
      13,    0,  120,    2, 0x06 /* Public */,
      14,    2,  121,    2, 0x06 /* Public */,
      17,    2,  126,    2, 0x06 /* Public */,
      20,    2,  131,    2, 0x06 /* Public */,
      21,    2,  136,    2, 0x06 /* Public */,
      22,    1,  141,    2, 0x06 /* Public */,
      23,    0,  144,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      24,    0,  145,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,   15,   16,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,   18,   19,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,   15,   16,
    QMetaType::Void, QMetaType::QReal, QMetaType::QReal,   18,   19,
    QMetaType::Void, QMetaType::QReal,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
      25, QMetaType::QColor, 0x00495103,
      26, QMetaType::QColor, 0x00495103,
      27, QMetaType::QColor, 0x00495103,
      28, QMetaType::QColor, 0x00495103,
      29, QMetaType::QColor, 0x00495103,
      30, QMetaType::QColor, 0x00495103,
      31, QMetaType::QColor, 0x00495103,
      32, QMetaType::QReal, 0x00495103,
      33, QMetaType::QString, 0x00495103,
      34, QMetaType::QString, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       6,
       7,
       8,
       9,
       4,
       5,

       0        // eod
};

void Waveform::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Waveform *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->backgroundColorChanged(); break;
        case 1: _t->foregroundColorChanged(); break;
        case 2: _t->axesColorChanged(); break;
        case 3: _t->textColorChanged(); break;
        case 4: _t->horizontalUnitsChanged(); break;
        case 5: _t->verticalUnitsChanged(); break;
        case 6: _t->selectionColorChanged(); break;
        case 7: _t->subSelectionColorChanged(); break;
        case 8: _t->envelopeColorChanged(); break;
        case 9: _t->sampleRateChanged(); break;
        case 10: _t->axesUpdated(); break;
        case 11: _t->selectionUpdated(); break;
        case 12: _t->horizontalRangeChanged((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 13: _t->verticalRangeChanged((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 14: _t->horizontalSelectionChanged((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 15: _t->verticalSelectionChanged((*reinterpret_cast< qreal(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 16: _t->hoverTime((*reinterpret_cast< qreal(*)>(_a[1]))); break;
        case 17: _t->waveViewChanged(); break;
        case 18: _t->onWaveViewChanges(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::backgroundColorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::foregroundColorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::axesColorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::textColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::horizontalUnitsChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::verticalUnitsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::selectionColorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::subSelectionColorChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::envelopeColorChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::sampleRateChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::axesUpdated)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::selectionUpdated)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Waveform::*)(qint64 , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::horizontalRangeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Waveform::*)(qreal , qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::verticalRangeChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Waveform::*)(qreal , qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::horizontalSelectionChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Waveform::*)(qreal , qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::verticalSelectionChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Waveform::*)(qreal );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::hoverTime)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Waveform::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Waveform::waveViewChanged)) {
                *result = 17;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Waveform *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBackgroundColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getForegroundColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getAxesColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getSelectionColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getSubSelectionColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getEnvelopeColor(); break;
        case 7: *reinterpret_cast< qreal*>(_v) = _t->getSampleRate(); break;
        case 8: *reinterpret_cast< QString*>(_v) = _t->getHorizontalUnits(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->getVerticalUnits(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Waveform *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setForegroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setAxesColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectionColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setSubSelectionColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setEnvelopeColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setSampleRate(*reinterpret_cast< qreal*>(_v)); break;
        case 8: _t->setHorizontalUnits(*reinterpret_cast< QString*>(_v)); break;
        case 9: _t->setVerticalUnits(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject Waveform::staticMetaObject = { {
    QMetaObject::SuperData::link<ThrottleableWidget::staticMetaObject>(),
    qt_meta_stringdata_Waveform.data,
    qt_meta_data_Waveform,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Waveform::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waveform::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Waveform.stringdata0))
        return static_cast<void*>(this);
    return ThrottleableWidget::qt_metacast(_clname);
}

int Waveform::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ThrottleableWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 10;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 10;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Waveform::backgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waveform::foregroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waveform::axesColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waveform::textColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waveform::horizontalUnitsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waveform::verticalUnitsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waveform::selectionColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waveform::subSelectionColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waveform::envelopeColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waveform::sampleRateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waveform::axesUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Waveform::selectionUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Waveform::horizontalRangeChanged(qint64 _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Waveform::verticalRangeChanged(qreal _t1, qreal _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void Waveform::horizontalSelectionChanged(qreal _t1, qreal _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void Waveform::verticalSelectionChanged(qreal _t1, qreal _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void Waveform::hoverTime(qreal _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void Waveform::waveViewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
