/****************************************************************************
** Meta object code from reading C++ file 'GLWaterfall.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../GLWaterfall.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GLWaterfall.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GLWaterfall_t {
    QByteArrayData data[41];
    char stringdata0[487];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GLWaterfall_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GLWaterfall_t qt_meta_stringdata_GLWaterfall = {
    {
QT_MOC_LITERAL(0, 0, 11), // "GLWaterfall"
QT_MOC_LITERAL(1, 12, 13), // "newCenterFreq"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 1), // "f"
QT_MOC_LITERAL(4, 29, 12), // "newDemodFreq"
QT_MOC_LITERAL(5, 42, 4), // "freq"
QT_MOC_LITERAL(6, 47, 5), // "delta"
QT_MOC_LITERAL(7, 53, 13), // "newLowCutFreq"
QT_MOC_LITERAL(8, 67, 14), // "newHighCutFreq"
QT_MOC_LITERAL(9, 82, 13), // "newModulation"
QT_MOC_LITERAL(10, 96, 10), // "modulation"
QT_MOC_LITERAL(11, 107, 13), // "newFilterFreq"
QT_MOC_LITERAL(12, 121, 3), // "low"
QT_MOC_LITERAL(13, 125, 4), // "high"
QT_MOC_LITERAL(14, 130, 21), // "pandapterRangeChanged"
QT_MOC_LITERAL(15, 152, 3), // "min"
QT_MOC_LITERAL(16, 156, 3), // "max"
QT_MOC_LITERAL(17, 160, 12), // "newZoomLevel"
QT_MOC_LITERAL(18, 173, 5), // "level"
QT_MOC_LITERAL(19, 179, 19), // "resetHorizontalZoom"
QT_MOC_LITERAL(20, 199, 16), // "moveToCenterFreq"
QT_MOC_LITERAL(21, 216, 15), // "moveToDemodFreq"
QT_MOC_LITERAL(22, 232, 11), // "zoomOnXAxis"
QT_MOC_LITERAL(23, 244, 15), // "setFftPlotColor"
QT_MOC_LITERAL(24, 260, 5), // "color"
QT_MOC_LITERAL(25, 266, 13), // "setFftBgColor"
QT_MOC_LITERAL(26, 280, 15), // "setFftAxesColor"
QT_MOC_LITERAL(27, 296, 15), // "setFftTextColor"
QT_MOC_LITERAL(28, 312, 17), // "setFilterBoxColor"
QT_MOC_LITERAL(29, 330, 17), // "setTimeStampColor"
QT_MOC_LITERAL(30, 348, 10), // "setFftFill"
QT_MOC_LITERAL(31, 359, 7), // "enabled"
QT_MOC_LITERAL(32, 367, 11), // "setPeakHold"
QT_MOC_LITERAL(33, 379, 11), // "setFftRange"
QT_MOC_LITERAL(34, 391, 17), // "setPandapterRange"
QT_MOC_LITERAL(35, 409, 17), // "setWaterfallRange"
QT_MOC_LITERAL(36, 427, 16), // "setPeakDetection"
QT_MOC_LITERAL(37, 444, 1), // "c"
QT_MOC_LITERAL(38, 446, 13), // "updateOverlay"
QT_MOC_LITERAL(39, 460, 18), // "setPercent2DScreen"
QT_MOC_LITERAL(40, 479, 7) // "percent"

    },
    "GLWaterfall\0newCenterFreq\0\0f\0newDemodFreq\0"
    "freq\0delta\0newLowCutFreq\0newHighCutFreq\0"
    "newModulation\0modulation\0newFilterFreq\0"
    "low\0high\0pandapterRangeChanged\0min\0"
    "max\0newZoomLevel\0level\0resetHorizontalZoom\0"
    "moveToCenterFreq\0moveToDemodFreq\0"
    "zoomOnXAxis\0setFftPlotColor\0color\0"
    "setFftBgColor\0setFftAxesColor\0"
    "setFftTextColor\0setFilterBoxColor\0"
    "setTimeStampColor\0setFftFill\0enabled\0"
    "setPeakHold\0setFftRange\0setPandapterRange\0"
    "setWaterfallRange\0setPeakDetection\0c\0"
    "updateOverlay\0setPercent2DScreen\0"
    "percent"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GLWaterfall[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x06 /* Public */,
       4,    2,  147,    2, 0x06 /* Public */,
       7,    1,  152,    2, 0x06 /* Public */,
       8,    1,  155,    2, 0x06 /* Public */,
       9,    1,  158,    2, 0x06 /* Public */,
      11,    2,  161,    2, 0x06 /* Public */,
      14,    2,  166,    2, 0x06 /* Public */,
      17,    1,  171,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      19,    0,  174,    2, 0x0a /* Public */,
      20,    0,  175,    2, 0x0a /* Public */,
      21,    0,  176,    2, 0x0a /* Public */,
      22,    1,  177,    2, 0x0a /* Public */,
      23,    1,  180,    2, 0x0a /* Public */,
      25,    1,  183,    2, 0x0a /* Public */,
      26,    1,  186,    2, 0x0a /* Public */,
      27,    1,  189,    2, 0x0a /* Public */,
      28,    1,  192,    2, 0x0a /* Public */,
      29,    1,  195,    2, 0x0a /* Public */,
      30,    1,  198,    2, 0x0a /* Public */,
      32,    1,  201,    2, 0x0a /* Public */,
      33,    2,  204,    2, 0x0a /* Public */,
      34,    2,  209,    2, 0x0a /* Public */,
      35,    2,  214,    2, 0x0a /* Public */,
      36,    2,  219,    2, 0x0a /* Public */,
      38,    0,  224,    2, 0x0a /* Public */,
      39,    1,  225,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::LongLong,    3,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,    5,    6,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   12,   13,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   15,   16,
    QMetaType::Void, QMetaType::Float,   18,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Float,   18,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::QColor,   24,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   15,   16,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   15,   16,
    QMetaType::Void, QMetaType::Float, QMetaType::Float,   15,   16,
    QMetaType::Void, QMetaType::Bool, QMetaType::Float,   31,   37,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   40,

       0        // eod
};

void GLWaterfall::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GLWaterfall *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newCenterFreq((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 1: _t->newDemodFreq((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 2: _t->newLowCutFreq((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->newHighCutFreq((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->newModulation((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->newFilterFreq((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->pandapterRangeChanged((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 7: _t->newZoomLevel((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 8: _t->resetHorizontalZoom(); break;
        case 9: _t->moveToCenterFreq(); break;
        case 10: _t->moveToDemodFreq(); break;
        case 11: _t->zoomOnXAxis((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 12: _t->setFftPlotColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 13: _t->setFftBgColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 14: _t->setFftAxesColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 15: _t->setFftTextColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 16: _t->setFilterBoxColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 17: _t->setTimeStampColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 18: _t->setFftFill((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->setPeakHold((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->setFftRange((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 21: _t->setPandapterRange((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 22: _t->setWaterfallRange((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 23: _t->setPeakDetection((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 24: _t->updateOverlay(); break;
        case 25: _t->setPercent2DScreen((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GLWaterfall::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newCenterFreq)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(qint64 , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newDemodFreq)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newLowCutFreq)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newHighCutFreq)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newModulation)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newFilterFreq)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(float , float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::pandapterRangeChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (GLWaterfall::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GLWaterfall::newZoomLevel)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject GLWaterfall::staticMetaObject = { {
    QMetaObject::SuperData::link<QOpenGLWidget::staticMetaObject>(),
    qt_meta_stringdata_GLWaterfall.data,
    qt_meta_data_GLWaterfall,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GLWaterfall::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GLWaterfall::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GLWaterfall.stringdata0))
        return static_cast<void*>(this);
    return QOpenGLWidget::qt_metacast(_clname);
}

int GLWaterfall::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QOpenGLWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void GLWaterfall::newCenterFreq(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void GLWaterfall::newDemodFreq(qint64 _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void GLWaterfall::newLowCutFreq(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void GLWaterfall::newHighCutFreq(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void GLWaterfall::newModulation(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void GLWaterfall::newFilterFreq(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void GLWaterfall::pandapterRangeChanged(float _t1, float _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void GLWaterfall::newZoomLevel(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
