/********************************************************************************
** Form generated from reading UI file 'TimeSpinBox.ui'
**
** Created by: Qt User Interface Compiler version 5.15.14
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIMESPINBOX_H
#define UI_TIMESPINBOX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_TimeSpinBox
{
public:
    QGridLayout *gridLayout;
    QDoubleSpinBox *valueSpin;
    QComboBox *unitCombo;

    void setupUi(QWidget *TimeSpinBox)
    {
        if (TimeSpinBox->objectName().isEmpty())
            TimeSpinBox->setObjectName(QString::fromUtf8("TimeSpinBox"));
        TimeSpinBox->resize(140, 24);
        gridLayout = new QGridLayout(TimeSpinBox);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        valueSpin = new QDoubleSpinBox(TimeSpinBox);
        valueSpin->setObjectName(QString::fromUtf8("valueSpin"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(valueSpin->sizePolicy().hasHeightForWidth());
        valueSpin->setSizePolicy(sizePolicy);
        valueSpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(valueSpin, 0, 0, 1, 1);

        unitCombo = new QComboBox(TimeSpinBox);
        unitCombo->addItem(QString());
        unitCombo->addItem(QString());
        unitCombo->addItem(QString());
        unitCombo->addItem(QString());
        unitCombo->setObjectName(QString::fromUtf8("unitCombo"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(unitCombo->sizePolicy().hasHeightForWidth());
        unitCombo->setSizePolicy(sizePolicy1);
        unitCombo->setEditable(false);
        unitCombo->setFrame(true);

        gridLayout->addWidget(unitCombo, 0, 1, 1, 1);


        retranslateUi(TimeSpinBox);

        QMetaObject::connectSlotsByName(TimeSpinBox);
    } // setupUi

    void retranslateUi(QWidget *TimeSpinBox)
    {
        TimeSpinBox->setWindowTitle(QCoreApplication::translate("TimeSpinBox", "Form", nullptr));
        unitCombo->setItemText(0, QCoreApplication::translate("TimeSpinBox", "s", nullptr));
        unitCombo->setItemText(1, QCoreApplication::translate("TimeSpinBox", "ms", nullptr));
        unitCombo->setItemText(2, QCoreApplication::translate("TimeSpinBox", "\302\265s", nullptr));
        unitCombo->setItemText(3, QCoreApplication::translate("TimeSpinBox", "ns", nullptr));

    } // retranslateUi

};

namespace Ui {
    class TimeSpinBox: public Ui_TimeSpinBox {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TIMESPINBOX_H
