/****************************************************************************
** Meta object code from reading C++ file 'LCD.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../LCD.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LCD.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LCD_t {
    QByteArrayData data[20];
    char stringdata0[233];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LCD_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LCD_t qt_meta_stringdata_LCD = {
    {
QT_MOC_LITERAL(0, 0, 3), // "LCD"
QT_MOC_LITERAL(1, 4, 12), // "valueChanged"
QT_MOC_LITERAL(2, 17, 0), // ""
QT_MOC_LITERAL(3, 18, 11), // "zoomChanged"
QT_MOC_LITERAL(4, 30, 16), // "thicknessChanged"
QT_MOC_LITERAL(5, 47, 15), // "segScaleChanged"
QT_MOC_LITERAL(6, 63, 22), // "backgroundColorChanged"
QT_MOC_LITERAL(7, 86, 22), // "foregroundColorChanged"
QT_MOC_LITERAL(8, 109, 10), // "maxChanged"
QT_MOC_LITERAL(9, 120, 10), // "minChanged"
QT_MOC_LITERAL(10, 131, 16), // "lockStateChanged"
QT_MOC_LITERAL(11, 148, 14), // "onTimerTimeout"
QT_MOC_LITERAL(12, 163, 5), // "value"
QT_MOC_LITERAL(13, 169, 8), // "segScale"
QT_MOC_LITERAL(14, 178, 9), // "thickness"
QT_MOC_LITERAL(15, 188, 4), // "zoom"
QT_MOC_LITERAL(16, 193, 15), // "backgroundColor"
QT_MOC_LITERAL(17, 209, 15), // "foregroundColor"
QT_MOC_LITERAL(18, 225, 3), // "min"
QT_MOC_LITERAL(19, 229, 3) // "max"

    },
    "LCD\0valueChanged\0\0zoomChanged\0"
    "thicknessChanged\0segScaleChanged\0"
    "backgroundColorChanged\0foregroundColorChanged\0"
    "maxChanged\0minChanged\0lockStateChanged\0"
    "onTimerTimeout\0value\0segScale\0thickness\0"
    "zoom\0backgroundColor\0foregroundColor\0"
    "min\0max"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LCD[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       8,   74, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    0,   65,    2, 0x06 /* Public */,
       4,    0,   66,    2, 0x06 /* Public */,
       5,    0,   67,    2, 0x06 /* Public */,
       6,    0,   68,    2, 0x06 /* Public */,
       7,    0,   69,    2, 0x06 /* Public */,
       8,    0,   70,    2, 0x06 /* Public */,
       9,    0,   71,    2, 0x06 /* Public */,
      10,    0,   72,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   73,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
      12, QMetaType::LongLong, 0x00495103,
      13, QMetaType::QReal, 0x00495103,
      14, QMetaType::QReal, 0x00495103,
      15, QMetaType::QReal, 0x00495103,
      16, QMetaType::QColor, 0x00495103,
      17, QMetaType::QColor, 0x00495103,
      18, QMetaType::LongLong, 0x00495103,
      19, QMetaType::LongLong, 0x00495103,

 // properties: notify_signal_id
       0,
       3,
       2,
       1,
       4,
       5,
       7,
       6,

       0        // eod
};

void LCD::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LCD *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged(); break;
        case 1: _t->zoomChanged(); break;
        case 2: _t->thicknessChanged(); break;
        case 3: _t->segScaleChanged(); break;
        case 4: _t->backgroundColorChanged(); break;
        case 5: _t->foregroundColorChanged(); break;
        case 6: _t->maxChanged(); break;
        case 7: _t->minChanged(); break;
        case 8: _t->lockStateChanged(); break;
        case 9: _t->onTimerTimeout(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::valueChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::zoomChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::thicknessChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::segScaleChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::backgroundColorChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::foregroundColorChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::maxChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::minChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (LCD::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LCD::lockStateChanged)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<LCD *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< qint64*>(_v) = _t->getValue(); break;
        case 1: *reinterpret_cast< qreal*>(_v) = _t->getSegScale(); break;
        case 2: *reinterpret_cast< qreal*>(_v) = _t->getThickness(); break;
        case 3: *reinterpret_cast< qreal*>(_v) = _t->getZoom(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getBackgroundColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getForegroundColor(); break;
        case 6: *reinterpret_cast< qint64*>(_v) = _t->getMin(); break;
        case 7: *reinterpret_cast< qint64*>(_v) = _t->getMax(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<LCD *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setValue(*reinterpret_cast< qint64*>(_v)); break;
        case 1: _t->setSegScale(*reinterpret_cast< qreal*>(_v)); break;
        case 2: _t->setThickness(*reinterpret_cast< qreal*>(_v)); break;
        case 3: _t->setZoom(*reinterpret_cast< qreal*>(_v)); break;
        case 4: _t->setBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setForegroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setMin(*reinterpret_cast< qint64*>(_v)); break;
        case 7: _t->setMax(*reinterpret_cast< qint64*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LCD::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_LCD.data,
    qt_meta_data_LCD,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LCD::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LCD::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LCD.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int LCD::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void LCD::valueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void LCD::zoomChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void LCD::thicknessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void LCD::segScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void LCD::backgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void LCD::foregroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void LCD::maxChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void LCD::minChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void LCD::lockStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
