/********************************************************************************
** Form generated from reading UI file 'FrequencySpinBox.ui'
**
** Created by: Qt User Interface Compiler version 5.15.14
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FREQUENCYSPINBOX_H
#define UI_FREQUENCYSPINBOX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FrequencySpinBox
{
public:
    QGridLayout *gridLayout;
    QDoubleSpinBox *frequencySpin;
    QPushButton *incFreqUnitsButton;
    QPushButton *decFreqUnitsButton;

    void setupUi(QWidget *FrequencySpinBox)
    {
        if (FrequencySpinBox->objectName().isEmpty())
            FrequencySpinBox->setObjectName(QString::fromUtf8("FrequencySpinBox"));
        FrequencySpinBox->resize(400, 33);
        gridLayout = new QGridLayout(FrequencySpinBox);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        frequencySpin = new QDoubleSpinBox(FrequencySpinBox);
        frequencySpin->setObjectName(QString::fromUtf8("frequencySpin"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(frequencySpin->sizePolicy().hasHeightForWidth());
        frequencySpin->setSizePolicy(sizePolicy);
        QFont font;
        font.setFamily(QString::fromUtf8("Monospace"));
        frequencySpin->setFont(font);
        frequencySpin->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        frequencySpin->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        frequencySpin->setProperty("showGroupSeparator", QVariant(true));
        frequencySpin->setDecimals(6);
        frequencySpin->setMaximum(4000000000.000000000000000);

        gridLayout->addWidget(frequencySpin, 0, 0, 1, 1);

        incFreqUnitsButton = new QPushButton(FrequencySpinBox);
        incFreqUnitsButton->setObjectName(QString::fromUtf8("incFreqUnitsButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(incFreqUnitsButton->sizePolicy().hasHeightForWidth());
        incFreqUnitsButton->setSizePolicy(sizePolicy1);
        incFreqUnitsButton->setMaximumSize(QSize(16777215, 16777215));

        gridLayout->addWidget(incFreqUnitsButton, 0, 1, 1, 1);

        decFreqUnitsButton = new QPushButton(FrequencySpinBox);
        decFreqUnitsButton->setObjectName(QString::fromUtf8("decFreqUnitsButton"));
        sizePolicy1.setHeightForWidth(decFreqUnitsButton->sizePolicy().hasHeightForWidth());
        decFreqUnitsButton->setSizePolicy(sizePolicy1);
        decFreqUnitsButton->setMaximumSize(QSize(16777215, 16777215));
        decFreqUnitsButton->setAutoFillBackground(false);
        decFreqUnitsButton->setFlat(false);

        gridLayout->addWidget(decFreqUnitsButton, 0, 2, 1, 1);


        retranslateUi(FrequencySpinBox);

        QMetaObject::connectSlotsByName(FrequencySpinBox);
    } // setupUi

    void retranslateUi(QWidget *FrequencySpinBox)
    {
        FrequencySpinBox->setWindowTitle(QCoreApplication::translate("FrequencySpinBox", "Form", nullptr));
        frequencySpin->setSuffix(QCoreApplication::translate("FrequencySpinBox", " MHz", nullptr));
        incFreqUnitsButton->setText(QCoreApplication::translate("FrequencySpinBox", "<", nullptr));
        decFreqUnitsButton->setText(QCoreApplication::translate("FrequencySpinBox", ">", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FrequencySpinBox: public Ui_FrequencySpinBox {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FREQUENCYSPINBOX_H
