/****************************************************************************
** Meta object code from reading C++ file 'Constellation.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../Constellation.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Constellation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Constellation_t {
    QByteArrayData data[11];
    char stringdata0[159];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Constellation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Constellation_t qt_meta_stringdata_Constellation = {
    {
QT_MOC_LITERAL(0, 0, 13), // "Constellation"
QT_MOC_LITERAL(1, 14, 16), // "orderHintChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 22), // "backgroundColorChanged"
QT_MOC_LITERAL(4, 55, 22), // "foregroundColorChanged"
QT_MOC_LITERAL(5, 78, 16), // "axesColorChanged"
QT_MOC_LITERAL(6, 95, 11), // "axesUpdated"
QT_MOC_LITERAL(7, 107, 9), // "orderHint"
QT_MOC_LITERAL(8, 117, 15), // "backgroundColor"
QT_MOC_LITERAL(9, 133, 15), // "foregroundColor"
QT_MOC_LITERAL(10, 149, 9) // "axesColor"

    },
    "Constellation\0orderHintChanged\0\0"
    "backgroundColorChanged\0foregroundColorChanged\0"
    "axesColorChanged\0axesUpdated\0orderHint\0"
    "backgroundColor\0foregroundColor\0"
    "axesColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Constellation[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       4,   44, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    0,   42,    2, 0x06 /* Public */,
       6,    0,   43,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       7, QMetaType::UInt, 0x00495103,
       8, QMetaType::QColor, 0x00495103,
       9, QMetaType::QColor, 0x00495103,
      10, QMetaType::QColor, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

       0        // eod
};

void Constellation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Constellation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->orderHintChanged(); break;
        case 1: _t->backgroundColorChanged(); break;
        case 2: _t->foregroundColorChanged(); break;
        case 3: _t->axesColorChanged(); break;
        case 4: _t->axesUpdated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Constellation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Constellation::orderHintChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Constellation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Constellation::backgroundColorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Constellation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Constellation::foregroundColorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Constellation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Constellation::axesColorChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Constellation::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Constellation::axesUpdated)) {
                *result = 4;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Constellation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< uint*>(_v) = _t->getOrderHint(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getBackgroundColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getForegroundColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getAxesColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Constellation *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOrderHint(*reinterpret_cast< uint*>(_v)); break;
        case 1: _t->setBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setForegroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setAxesColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Constellation::staticMetaObject = { {
    QMetaObject::SuperData::link<ThrottleableWidget::staticMetaObject>(),
    qt_meta_stringdata_Constellation.data,
    qt_meta_data_Constellation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Constellation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Constellation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Constellation.stringdata0))
        return static_cast<void*>(this);
    return ThrottleableWidget::qt_metacast(_clname);
}

int Constellation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ThrottleableWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void Constellation::orderHintChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Constellation::backgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Constellation::foregroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Constellation::axesColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Constellation::axesUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
