/****************************************************************************
** Meta object code from reading C++ file 'SymView.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../SymView.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'SymView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SymView_t {
    QByteArrayData data[14];
    char stringdata0[170];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SymView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SymView_t qt_meta_stringdata_SymView = {
    {
QT_MOC_LITERAL(0, 0, 7), // "SymView"
QT_MOC_LITERAL(1, 8, 13), // "offsetChanged"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 14), // "hOffsetChanged"
QT_MOC_LITERAL(4, 38, 13), // "strideChanged"
QT_MOC_LITERAL(5, 52, 11), // "zoomChanged"
QT_MOC_LITERAL(6, 64, 11), // "hoverSymbol"
QT_MOC_LITERAL(7, 76, 8), // "position"
QT_MOC_LITERAL(8, 85, 22), // "backgroundColorChanged"
QT_MOC_LITERAL(9, 108, 14), // "loColorChanged"
QT_MOC_LITERAL(10, 123, 14), // "hiColorChanged"
QT_MOC_LITERAL(11, 138, 15), // "backgroundColor"
QT_MOC_LITERAL(12, 154, 7), // "loColor"
QT_MOC_LITERAL(13, 162, 7) // "hiColor"

    },
    "SymView\0offsetChanged\0\0hOffsetChanged\0"
    "strideChanged\0zoomChanged\0hoverSymbol\0"
    "position\0backgroundColorChanged\0"
    "loColorChanged\0hiColorChanged\0"
    "backgroundColor\0loColor\0hiColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SymView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       3,   72, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       3,    1,   57,    2, 0x06 /* Public */,
       4,    1,   60,    2, 0x06 /* Public */,
       5,    1,   63,    2, 0x06 /* Public */,
       6,    1,   66,    2, 0x06 /* Public */,
       8,    0,   69,    2, 0x06 /* Public */,
       9,    0,   70,    2, 0x06 /* Public */,
      10,    0,   71,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::UInt,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::UInt,    2,
    QMetaType::Void, QMetaType::UInt,    2,
    QMetaType::Void, QMetaType::UInt,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      11, QMetaType::QColor, 0x00495103,
      12, QMetaType::QColor, 0x00495103,
      13, QMetaType::QColor, 0x00495103,

 // properties: notify_signal_id
       5,
       6,
       7,

       0        // eod
};

void SymView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SymView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->offsetChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 1: _t->hOffsetChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->strideChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 3: _t->zoomChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 4: _t->hoverSymbol((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 5: _t->backgroundColorChanged(); break;
        case 6: _t->loColorChanged(); break;
        case 7: _t->hiColorChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SymView::*)(unsigned int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::offsetChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SymView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::hOffsetChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SymView::*)(unsigned int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::strideChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SymView::*)(unsigned int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::zoomChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SymView::*)(unsigned int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::hoverSymbol)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (SymView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::backgroundColorChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (SymView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::loColorChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (SymView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymView::hiColorChanged)) {
                *result = 7;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SymView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBackgroundColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getLoColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getHiColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<SymView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBackgroundColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setLoColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setHiColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject SymView::staticMetaObject = { {
    QMetaObject::SuperData::link<ThrottleableWidget::staticMetaObject>(),
    qt_meta_stringdata_SymView.data,
    qt_meta_data_SymView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SymView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SymView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SymView.stringdata0))
        return static_cast<void*>(this);
    return ThrottleableWidget::qt_metacast(_clname);
}

int SymView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ThrottleableWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void SymView::offsetChanged(unsigned int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SymView::hOffsetChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void SymView::strideChanged(unsigned int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void SymView::zoomChanged(unsigned int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void SymView::hoverSymbol(unsigned int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void SymView::backgroundColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SymView::loColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void SymView::hiColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
