/********************************************************************************
** Form generated from reading UI file 'ColorChooserButton.ui'
**
** Created by: Qt User Interface Compiler version 5.15.14
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLORCHOOSERBUTTON_H
#define UI_COLORCHOOSERBUTTON_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ColorChooserButton
{
public:
    QGridLayout *gridLayout;
    QPushButton *pushButton;

    void setupUi(QWidget *ColorChooserButton)
    {
        if (ColorChooserButton->objectName().isEmpty())
            ColorChooserButton->setObjectName(QString::fromUtf8("ColorChooserButton"));
        ColorChooserButton->resize(82, 28);
        gridLayout = new QGridLayout(ColorChooserButton);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        pushButton = new QPushButton(ColorChooserButton);
        pushButton->setObjectName(QString::fromUtf8("pushButton"));

        gridLayout->addWidget(pushButton, 0, 0, 1, 1);


        retranslateUi(ColorChooserButton);

        QMetaObject::connectSlotsByName(ColorChooserButton);
    } // setupUi

    void retranslateUi(QWidget *ColorChooserButton)
    {
        ColorChooserButton->setWindowTitle(QCoreApplication::translate("ColorChooserButton", "Form", nullptr));
        pushButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class ColorChooserButton: public Ui_ColorChooserButton {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COLORCHOOSERBUTTON_H
